'''pkg_classify.py: classify Ubuntu source packages into different sections
like "gnome", "kde", and "" (which means "all the rest").'''

import apt_pkg, urllib, gzip, tempfile, os

from macros import _file_map

def package_list_map(url, merge_to = None):
    '''Read a package list from the given URL and parse it into a map 
    package->key->value.

    E. g. package_list_map('Sources.gz')['dpkg']['Version'] would deliver the
    Version of the dpkg source package.'''

    # write the uncompressed downloaded list into a temporary file
    downloadfile = urllib.urlretrieve(url, tempfile.mkstemp()[1])[0]
    list = tempfile.TemporaryFile()
    try:
        print >> list, gzip.open(downloadfile).read()
    except IOError:
        print >> list, open(downloadfile).read()
    os.unlink(downloadfile)
    list.seek(0)

    if merge_to is None:
        map = {}
    else:
        map = merge_to
    parser = apt_pkg.ParseTagFile(list)
    while parser.Step() == True:
        pkg = {}
        for key in parser.Section.keys():
            if key != 'Package':
                pkg[key] = parser.Section.get(key)
        map[parser.Section.get('Package')] = pkg
    return map

class PackageClassificator:
    def __init__(self, release, domain_map, archive_root = 'http://archive.ubuntu.com/ubuntu'):
        # read domain_map.txt
        self.domain_package = {}
        for line in domain_map.splitlines():
            f = line.split()
            if len(f) != 2:
                continue
            self.domain_package[f[1]] = f[0]

        release = release.split('-')[0]

        # get archive map
        self.package_info = package_list_map(
            '%s/dists/%s/main/source/Sources.gz' % (archive_root, release))
        self.package_info = package_list_map(
            '%s/dists/%s/restricted/source/Sources.gz' % (archive_root, release), self.package_info)

    def classify_domain(self, domain):
        '''Classify given domain.

        Raises an exception if the domain is unknown.'''

        try:
            pkg = self.domain_package[domain]
        except KeyError:
            raise KeyError, 'Unknown domain %s' % domain
        return self.classify_package(pkg)

    def classify_package(self, package):
        '''Classify given source package.

        Raises an exception if the package is unknown.'''

        try:
            return _file_map('maps/classify-overrides', package, ':')
        except KeyError:
            pass

        i = self.package_info[package]
        desc = (package + i['Binary'] + i['Section'] + i['Maintainer'] + 
            i.get('Build-Depends', '')).lower()
        has_gnome = 'gnome' in desc or 'gtk' in i['Binary'] or package.startswith('gtk')
        has_kde = (desc.find('kde') >= 0)
        if has_gnome and not has_kde:
            return 'gnome'
        elif has_kde and not has_gnome:
            return 'kde'
        else:
            return ''

# test

if __name__ == '__main__':
    testmap = """
gnome-session gnome-session-2.0
gnome-menus gnome-menus
gtk+1.2 gtk+
gtkspell gtkspell
gaim gaim
knetworkconf knetworkconfmodule
postgresql-8.0 pg_dump-8.0
postgresql-8.1 pg_dump-8.1
software-properties software-properties
"""
    print '*** dapper ***'
    pc = PackageClassificator('dapper', testmap)
    print 'domain gnome-session-2.0 (expected: gnome):', pc.classify_domain('gnome-session-2.0')
    print 'package gnome-session (expected: gnome):', pc.classify_package('gnome-session')
    print 'domain gtk+ (expected: gnome):', pc.classify_domain('gtk+')
    print 'domain gtkspell (expected: gnome):', pc.classify_domain('gtkspell')
    print 'domain gaim (expected: empty):', pc.classify_domain('gaim')
    print 'package meta-kde (expected: kde):', pc.classify_package('meta-kde')
    print 'domain pg_dump-8.1 (expected: empty):', pc.classify_domain('pg_dump-8.1')
    print 'package postgresql-8.1 (expected: empty):', pc.classify_package('postgresql-8.1')
    print 'package gnome-menus (expected: gnome):', pc.classify_package('gnome-menus')

    print '*** feisty ***'
    pc = PackageClassificator('feisty', testmap)
    print 'domain software-properties (expected: empty):', pc.classify_domain('software-properties')

    print '*** gutsy ***'
    pc = PackageClassificator('gutsy', testmap)
    print 'package gdebi (expected: empty):', pc.classify_package('gdebi')
    print 'package restricted-manager (expected: empty):', pc.classify_package('restricted-manager')
